@@ -13,28 +13,25 @@ See the examples directory to learn about the usage.
 Probably, to get started, you might also want to have a look at
 `prompt_toolkit.shortcuts.prompt`.
 """
-from __future__ import unicode_literals
 from .application import Application
-from .shortcuts import PromptSession, prompt, print_formatted_text
-from .formatted_text import HTML, ANSI
-
+from .formatted_text import ANSI, HTML
+from .shortcuts import PromptSession, print_formatted_text, prompt
 
 # Don't forget to update in `docs/conf.py`!
-__version__ = '2.0.9'
+__version__ = "3.0.2"
 
 # Version tuple.
-VERSION = tuple(__version__.split('.'))
+VERSION = tuple(__version__.split("."))
 
 
 __all__ = [
     # Application.
-    'Application',
-
+    "Application",
     # Shortcuts.
-    'prompt',
-    'PromptSession',
-    'print_formatted_text',
-
+    "prompt",
+    "PromptSession",
+    "print_formatted_text",
     # Formatted text.
-    'HTML', 'ANSI',
+    "HTML",
+    "ANSI",
 ]
