@@ -1,20 +1,32 @@
-from __future__ import unicode_literals
-
-from ..eventloop import get_event_loop
-from .base import Input
-from .posix_utils import PosixStdinReader
-from .vt100_parser import Vt100Parser
+import contextlib
 import io
 import os
 import sys
 import termios
 import tty
-import contextlib
+from asyncio import AbstractEventLoop, get_event_loop
+from typing import (
+    Callable,
+    ContextManager,
+    Dict,
+    Generator,
+    List,
+    Optional,
+    Set,
+    TextIO,
+    Tuple,
+    Union,
+)
+
+from ..key_binding import KeyPress
+from .base import Input
+from .posix_utils import PosixStdinReader
+from .vt100_parser import Vt100Parser
 
 __all__ = [
-    'Vt100Input',
-    'raw_mode',
-    'cooked_mode',
+    "Vt100Input",
+    "raw_mode",
+    "cooked_mode",
 ]
 
 
@@ -23,21 +35,24 @@ class Vt100Input(Input):
     Vt100 input for Posix systems.
     (This uses a posix file descriptor that can be registered in the event loop.)
     """
-    _fds_not_a_terminal = set()  # For the error messages. Only display "Input
-                                 # is not a terminal" once per file descriptor.
 
-    def __init__(self, stdin):
+    # For the error messages. Only display "Input is not a terminal" once per
+    # file descriptor.
+    _fds_not_a_terminal: Set[int] = set()
+
+    def __init__(self, stdin: TextIO) -> None:
         # Test whether the given input object has a file descriptor.
         # (Idle reports stdin to be a TTY, but fileno() is not implemented.)
         try:
             # This should not raise, but can return 0.
             stdin.fileno()
         except io.UnsupportedOperation:
-            if 'idlelib.run' in sys.modules:
+            if "idlelib.run" in sys.modules:
                 raise io.UnsupportedOperation(
-                    'Stdin is not a terminal. Running from Idle is not supported.')
+                    "Stdin is not a terminal. Running from Idle is not supported."
+                )
             else:
-                raise io.UnsupportedOperation('Stdin is not a terminal.')
+                raise io.UnsupportedOperation("Stdin is not a terminal.")
 
         # Even when we have a file descriptor, it doesn't mean it's a TTY.
         # Normally, this requires a real TTY device, but people instantiate
@@ -48,7 +63,7 @@ class Vt100Input(Input):
         fd = stdin.fileno()
 
         if not isatty and fd not in Vt100Input._fds_not_a_terminal:
-            msg = 'Warning: Input is not to a terminal (fd=%r).\n'
+            msg = "Warning: Input is not a terminal (fd=%r).\n"
             sys.stderr.write(msg % fd)
             Vt100Input._fds_not_a_terminal.add(fd)
 
@@ -59,33 +74,38 @@ class Vt100Input(Input):
         # underlying file is closed, so that `typeahead_hash()` keeps working.
         self._fileno = stdin.fileno()
 
-        self._buffer = []  # Buffer to collect the Key objects.
+        self._buffer: List[KeyPress] = []  # Buffer to collect the Key objects.
         self.stdin_reader = PosixStdinReader(self._fileno)
         self.vt100_parser = Vt100Parser(
-            lambda key: self._buffer.append(key))
+            lambda key_press: self._buffer.append(key_press)
+        )
 
     @property
-    def responds_to_cpr(self):
+    def responds_to_cpr(self) -> bool:
         # When the input is a tty, we assume that CPR is supported.
         # It's not when the input is piped from Pexpect.
-        return self.stdin.isatty()
+        if os.environ.get("PROMPT_TOOLKIT_NO_CPR", "") == "1":
+            return False
+        try:
+            return self.stdin.isatty()
+        except ValueError:
+            return False  # ValueError: I/O operation on closed file
 
-    def attach(self, input_ready_callback):
+    def attach(self, input_ready_callback: Callable[[], None]) -> ContextManager[None]:
         """
         Return a context manager that makes this input active in the current
         event loop.
         """
-        assert callable(input_ready_callback)
         return _attached_input(self, input_ready_callback)
 
-    def detach(self):
+    def detach(self) -> ContextManager[None]:
         """
         Return a context manager that makes sure that this input is not active
         in the current event loop.
         """
         return _detached_input(self)
 
-    def read_keys(self):
+    def read_keys(self) -> List[KeyPress]:
         " Read list of KeyPress. "
         # Read text from stdin.
         data = self.stdin_reader.read()
@@ -98,7 +118,7 @@ class Vt100Input(Input):
         self._buffer = []
         return result
 
-    def flush_keys(self):
+    def flush_keys(self) -> List[KeyPress]:
         """
         Flush pending keys and return them.
         (Used for flushing the 'escape' key.)
@@ -113,27 +133,31 @@ class Vt100Input(Input):
         return result
 
     @property
-    def closed(self):
+    def closed(self) -> bool:
         return self.stdin_reader.closed
 
-    def raw_mode(self):
+    def raw_mode(self) -> ContextManager[None]:
         return raw_mode(self.stdin.fileno())
 
-    def cooked_mode(self):
+    def cooked_mode(self) -> ContextManager[None]:
         return cooked_mode(self.stdin.fileno())
 
-    def fileno(self):
+    def fileno(self) -> int:
         return self.stdin.fileno()
 
-    def typeahead_hash(self):
-        return 'fd-%s' % (self._fileno, )
+    def typeahead_hash(self) -> str:
+        return "fd-%s" % (self._fileno,)
 
 
-_current_callbacks = {}  # (loop, fd) -> current callback
+_current_callbacks: Dict[
+    Tuple[AbstractEventLoop, int], Optional[Callable[[], None]]
+] = {}  # (loop, fd) -> current callback
 
 
 @contextlib.contextmanager
-def _attached_input(input, callback):
+def _attached_input(
+    input: Vt100Input, callback: Callable[[], None]
+) -> Generator[None, None, None]:
     """
     Context manager that makes this input active in the current event loop.
 
@@ -160,7 +184,7 @@ def _attached_input(input, callback):
 
 
 @contextlib.contextmanager
-def _detached_input(input):
+def _detached_input(input: Vt100Input) -> Generator[None, None, None]:
     loop = get_event_loop()
     fd = input.fileno()
     previous = _current_callbacks.get((loop, fd))
@@ -177,7 +201,7 @@ def _detached_input(input):
             _current_callbacks[loop, fd] = previous
 
 
-class raw_mode(object):
+class raw_mode:
     """
     ::
 
@@ -186,6 +210,7 @@ class raw_mode(object):
 
     We ignore errors when executing `tcgetattr` fails.
     """
+
     # There are several reasons for ignoring errors:
     # 1. To avoid the "Inappropriate ioctl for device" crash if somebody would
     #    execute this code (In a Python REPL, for instance):
@@ -200,15 +225,16 @@ class raw_mode(object):
 
     # 2. Related, when stdin is an SSH pipe, and no full terminal was allocated.
     #    See: https://github.com/jonathanslenders/python-prompt-toolkit/pull/165
-    def __init__(self, fileno):
+    def __init__(self, fileno: int) -> None:
         self.fileno = fileno
+        self.attrs_before: Optional[List[Union[int, List[bytes]]]]
         try:
             self.attrs_before = termios.tcgetattr(fileno)
         except termios.error:
             # Ignore attribute errors.
             self.attrs_before = None
 
-    def __enter__(self):
+    def __enter__(self) -> None:
         # NOTE: On os X systems, using pty.setraw() fails. Therefor we are using this:
         try:
             newattr = termios.tcgetattr(self.fileno)
@@ -223,12 +249,12 @@ class raw_mode(object):
             # Solaris and derived operating systems it defaults to 4. (This is
             # because the VMIN slot is the same as the VEOF slot, which
             # defaults to ASCII EOT = Ctrl-D = 4.)
-            newattr[tty.CC][termios.VMIN] = 1
+            newattr[tty.CC][termios.VMIN] = 1  # type: ignore
 
             termios.tcsetattr(self.fileno, termios.TCSANOW, newattr)
 
             # Put the terminal in cursor mode. (Instead of application mode.)
-            os.write(self.fileno, b'\x1b[?1l')
+            os.write(self.fileno, b"\x1b[?1l")
 
     @classmethod
     def _patch_lflag(cls, attrs):
@@ -240,13 +266,16 @@ class raw_mode(object):
             # Disable XON/XOFF flow control on output and input.
             # (Don't capture Ctrl-S and Ctrl-Q.)
             # Like executing: "stty -ixon."
-            termios.IXON | termios.IXOFF |
-
+            termios.IXON
+            | termios.IXOFF
+            |
             # Don't translate carriage return into newline on input.
-            termios.ICRNL | termios.INLCR | termios.IGNCR
+            termios.ICRNL
+            | termios.INLCR
+            | termios.IGNCR
         )
 
-    def __exit__(self, *a, **kw):
+    def __exit__(self, *a: object) -> None:
         if self.attrs_before is not None:
             try:
                 termios.tcsetattr(self.fileno, termios.TCSANOW, self.attrs_before)
@@ -265,6 +294,7 @@ class cooked_mode(raw_mode):
         with cooked_mode(stdin):
             ''' the pseudo-terminal stdin is now used in cooked mode. '''
     """
+
     @classmethod
     def _patch_lflag(cls, attrs):
         return attrs | (termios.ECHO | termios.ICANON | termios.IEXTEN | termios.ISIG)
