@@ -1,38 +1,60 @@
-from __future__ import unicode_literals
-from ctypes import windll, pointer
-from ctypes.wintypes import DWORD
-from six.moves import range
+import msvcrt
+import os
+import sys
+from abc import abstractmethod
+from asyncio import get_event_loop
 from contextlib import contextmanager
+from ctypes import pointer, windll
+from ctypes.wintypes import DWORD, HANDLE
+from typing import Callable, ContextManager, Dict, Iterable, Optional
 
-from .ansi_escape_sequences import REVERSE_ANSI_SEQUENCES
-from .base import Input
-from prompt_toolkit.eventloop import get_event_loop
+from prompt_toolkit.eventloop import run_in_executor_with_context
 from prompt_toolkit.eventloop.win32 import wait_for_handles
 from prompt_toolkit.key_binding.key_processor import KeyPress
 from prompt_toolkit.keys import Keys
 from prompt_toolkit.mouse_events import MouseEventType
-from prompt_toolkit.win32_types import EventTypes, KEY_EVENT_RECORD, MOUSE_EVENT_RECORD, INPUT_RECORD, STD_INPUT_HANDLE
+from prompt_toolkit.win32_types import (
+    INPUT_RECORD,
+    KEY_EVENT_RECORD,
+    MOUSE_EVENT_RECORD,
+    STD_INPUT_HANDLE,
+    EventTypes,
+)
 
-import msvcrt
-import os
-import sys
-import six
+from .ansi_escape_sequences import REVERSE_ANSI_SEQUENCES
+from .base import Input
 
 __all__ = [
-    'Win32Input',
-    'ConsoleInputReader',
-    'raw_mode',
-    'cooked_mode',
-    'attach_win32_input',
-    'detach_win32_input',
+    "Win32Input",
+    "ConsoleInputReader",
+    "raw_mode",
+    "cooked_mode",
+    "attach_win32_input",
+    "detach_win32_input",
 ]
 
 
-class Win32Input(Input):
+class _Win32InputBase(Input):
+    """
+    Base class for `Win32Input` and `Win32PipeInput`.
+    """
+
+    def __init__(self) -> None:
+        self.win32_handles = _Win32Handles()
+
+    @property
+    @abstractmethod
+    def handle(self) -> HANDLE:
+        pass
+
+
+class Win32Input(_Win32InputBase):
     """
     `Input` class that reads from the Windows console.
     """
+
     def __init__(self, stdin=None):
+        super().__init__()
         self.console_input_reader = ConsoleInputReader()
 
     def attach(self, input_ready_callback):
@@ -43,7 +65,7 @@ class Win32Input(Input):
         assert callable(input_ready_callback)
         return attach_win32_input(self, input_ready_callback)
 
-    def detach(self):
+    def detach(self) -> ContextManager[None]:
         """
         Return a context manager that makes sure that this input is not active
         in the current event loop.
@@ -53,79 +75,78 @@ class Win32Input(Input):
     def read_keys(self):
         return list(self.console_input_reader.read())
 
-    def flush(self):
+    def flush(self) -> None:
         pass
 
     @property
-    def closed(self):
+    def closed(self) -> bool:
         return False
 
-    def raw_mode(self):
+    def raw_mode(self) -> ContextManager[None]:
         return raw_mode()
 
-    def cooked_mode(self):
+    def cooked_mode(self) -> ContextManager[None]:
         return cooked_mode()
 
-    def fileno(self):
+    def fileno(self) -> int:
         # The windows console doesn't depend on the file handle, so
         # this is not used for the event loop (which uses the
         # handle instead). But it's used in `Application.run_system_command`
         # which opens a subprocess with a given stdin/stdout.
         return sys.stdin.fileno()
 
-    def typeahead_hash(self):
-        return 'win32-input'
+    def typeahead_hash(self) -> str:
+        return "win32-input"
 
-    def close(self):
+    def close(self) -> None:
         self.console_input_reader.close()
 
     @property
-    def handle(self):
+    def handle(self) -> HANDLE:
         return self.console_input_reader.handle
 
 
-class ConsoleInputReader(object):
+class ConsoleInputReader:
     """
     :param recognize_paste: When True, try to discover paste actions and turn
         the event into a BracketedPaste.
     """
+
     # Keys with character data.
     mappings = {
-        b'\x1b': Keys.Escape,
-
-        b'\x00': Keys.ControlSpace,  # Control-Space (Also for Ctrl-@)
-        b'\x01': Keys.ControlA,  # Control-A (home)
-        b'\x02': Keys.ControlB,  # Control-B (emacs cursor left)
-        b'\x03': Keys.ControlC,  # Control-C (interrupt)
-        b'\x04': Keys.ControlD,  # Control-D (exit)
-        b'\x05': Keys.ControlE,  # Control-E (end)
-        b'\x06': Keys.ControlF,  # Control-F (cursor forward)
-        b'\x07': Keys.ControlG,  # Control-G
-        b'\x08': Keys.ControlH,  # Control-H (8) (Identical to '\b')
-        b'\x09': Keys.ControlI,  # Control-I (9) (Identical to '\t')
-        b'\x0a': Keys.ControlJ,  # Control-J (10) (Identical to '\n')
-        b'\x0b': Keys.ControlK,  # Control-K (delete until end of line; vertical tab)
-        b'\x0c': Keys.ControlL,  # Control-L (clear; form feed)
-        b'\x0d': Keys.ControlM,  # Control-M (enter)
-        b'\x0e': Keys.ControlN,  # Control-N (14) (history forward)
-        b'\x0f': Keys.ControlO,  # Control-O (15)
-        b'\x10': Keys.ControlP,  # Control-P (16) (history back)
-        b'\x11': Keys.ControlQ,  # Control-Q
-        b'\x12': Keys.ControlR,  # Control-R (18) (reverse search)
-        b'\x13': Keys.ControlS,  # Control-S (19) (forward search)
-        b'\x14': Keys.ControlT,  # Control-T
-        b'\x15': Keys.ControlU,  # Control-U
-        b'\x16': Keys.ControlV,  # Control-V
-        b'\x17': Keys.ControlW,  # Control-W
-        b'\x18': Keys.ControlX,  # Control-X
-        b'\x19': Keys.ControlY,  # Control-Y (25)
-        b'\x1a': Keys.ControlZ,  # Control-Z
-
-        b'\x1c': Keys.ControlBackslash,  # Both Control-\ and Ctrl-|
-        b'\x1d': Keys.ControlSquareClose,  # Control-]
-        b'\x1e': Keys.ControlCircumflex,  # Control-^
-        b'\x1f': Keys.ControlUnderscore,  # Control-underscore (Also for Ctrl-hyphen.)
-        b'\x7f': Keys.Backspace,  # (127) Backspace   (ASCII Delete.)
+        b"\x1b": Keys.Escape,
+        b"\x00": Keys.ControlSpace,  # Control-Space (Also for Ctrl-@)
+        b"\x01": Keys.ControlA,  # Control-A (home)
+        b"\x02": Keys.ControlB,  # Control-B (emacs cursor left)
+        b"\x03": Keys.ControlC,  # Control-C (interrupt)
+        b"\x04": Keys.ControlD,  # Control-D (exit)
+        b"\x05": Keys.ControlE,  # Control-E (end)
+        b"\x06": Keys.ControlF,  # Control-F (cursor forward)
+        b"\x07": Keys.ControlG,  # Control-G
+        b"\x08": Keys.ControlH,  # Control-H (8) (Identical to '\b')
+        b"\x09": Keys.ControlI,  # Control-I (9) (Identical to '\t')
+        b"\x0a": Keys.ControlJ,  # Control-J (10) (Identical to '\n')
+        b"\x0b": Keys.ControlK,  # Control-K (delete until end of line; vertical tab)
+        b"\x0c": Keys.ControlL,  # Control-L (clear; form feed)
+        b"\x0d": Keys.ControlM,  # Control-M (enter)
+        b"\x0e": Keys.ControlN,  # Control-N (14) (history forward)
+        b"\x0f": Keys.ControlO,  # Control-O (15)
+        b"\x10": Keys.ControlP,  # Control-P (16) (history back)
+        b"\x11": Keys.ControlQ,  # Control-Q
+        b"\x12": Keys.ControlR,  # Control-R (18) (reverse search)
+        b"\x13": Keys.ControlS,  # Control-S (19) (forward search)
+        b"\x14": Keys.ControlT,  # Control-T
+        b"\x15": Keys.ControlU,  # Control-U
+        b"\x16": Keys.ControlV,  # Control-V
+        b"\x17": Keys.ControlW,  # Control-W
+        b"\x18": Keys.ControlX,  # Control-X
+        b"\x19": Keys.ControlY,  # Control-Y (25)
+        b"\x1a": Keys.ControlZ,  # Control-Z
+        b"\x1c": Keys.ControlBackslash,  # Both Control-\ and Ctrl-|
+        b"\x1d": Keys.ControlSquareClose,  # Control-]
+        b"\x1e": Keys.ControlCircumflex,  # Control-^
+        b"\x1f": Keys.ControlUnderscore,  # Control-underscore (Also for Ctrl-hyphen.)
+        b"\x7f": Keys.Backspace,  # (127) Backspace   (ASCII Delete.)
     }
 
     # Keys that don't carry character data.
@@ -135,16 +156,13 @@ class ConsoleInputReader(object):
         34: Keys.PageDown,
         35: Keys.End,
         36: Keys.Home,
-
         # Arrows
         37: Keys.Left,
         38: Keys.Up,
         39: Keys.Right,
         40: Keys.Down,
-
         45: Keys.Insert,
         46: Keys.Delete,
-
         # F-keys.
         112: Keys.F1,
         113: Keys.F2,
@@ -166,24 +184,25 @@ class ConsoleInputReader(object):
     LEFT_CTRL_PRESSED = 0x0008
     RIGHT_CTRL_PRESSED = 0x0004
 
-    def __init__(self, recognize_paste=True):
+    def __init__(self, recognize_paste: bool = True) -> None:
         self._fdcon = None
         self.recognize_paste = recognize_paste
 
         # When stdin is a tty, use that handle, otherwise, create a handle from
         # CONIN$.
+        self.handle: HANDLE
         if sys.stdin.isatty():
-            self.handle = windll.kernel32.GetStdHandle(STD_INPUT_HANDLE)
+            self.handle = HANDLE(windll.kernel32.GetStdHandle(STD_INPUT_HANDLE))
         else:
-            self._fdcon = os.open('CONIN$', os.O_RDWR | os.O_BINARY)
-            self.handle = msvcrt.get_osfhandle(self._fdcon)
+            self._fdcon = os.open("CONIN$", os.O_RDWR | os.O_BINARY)
+            self.handle = HANDLE(msvcrt.get_osfhandle(self._fdcon))
 
-    def close(self):
+    def close(self) -> None:
         " Close fdcon. "
         if self._fdcon is not None:
             os.close(self._fdcon)
 
-    def read(self):
+    def read(self) -> Iterable[KeyPress]:
         """
         Return a list of `KeyPress` instances. It won't return anything when
         there was nothing to read.  (This function doesn't block.)
@@ -207,7 +226,8 @@ class ConsoleInputReader(object):
 
         # Get next batch of input event.
         windll.kernel32.ReadConsoleInputW(
-            self.handle, pointer(input_records), max_count, pointer(read))
+            self.handle, pointer(input_records), max_count, pointer(read)
+        )
 
         # First, get all the keys from the input buffer, in order to determine
         # whether we should consider this a paste event or not.
@@ -218,12 +238,13 @@ class ConsoleInputReader(object):
 
         if self.recognize_paste and self._is_paste(all_keys):
             gen = iter(all_keys)
+            k: Optional[KeyPress]
+
             for k in gen:
                 # Pasting: if the current key consists of text or \n, turn it
                 # into a BracketedPaste.
                 data = []
-                while k and (isinstance(k.key, six.text_type) or
-                             k.key == Keys.ControlJ):
+                while k and (isinstance(k.key, str) or k.key == Keys.ControlJ):
                     data.append(k.data)
                     try:
                         k = next(gen)
@@ -231,21 +252,25 @@ class ConsoleInputReader(object):
                         k = None
 
                 if data:
-                    yield KeyPress(Keys.BracketedPaste, ''.join(data))
+                    yield KeyPress(Keys.BracketedPaste, "".join(data))
                 if k is not None:
                     yield k
         else:
-            for k in all_keys:
-                yield k
+            for k2 in all_keys:
+                yield k2
 
-    def _insert_key_data(self, key_press):
+    def _insert_key_data(self, key_press: KeyPress) -> KeyPress:
         """
         Insert KeyPress data, for vt100 compatibility.
         """
         if key_press.data:
             return key_press
 
-        data = REVERSE_ANSI_SEQUENCES.get(key_press.key, '')
+        if isinstance(key_press.key, Keys):
+            data = REVERSE_ANSI_SEQUENCES.get(key_press.key, "")
+        else:
+            data = ""
+
         return KeyPress(key_press.key, data)
 
     def _get_keys(self, read, input_records):
@@ -273,7 +298,7 @@ class ConsoleInputReader(object):
                         yield key_press
 
     @staticmethod
-    def _is_paste(keys):
+    def _is_paste(keys) -> bool:
         """
         Return `True` when we should consider this list of keys as a paste
         event. Pasted text on windows will be turned into a
@@ -287,7 +312,7 @@ class ConsoleInputReader(object):
         newline_count = 0
 
         for k in keys:
-            if isinstance(k.key, six.text_type):
+            if isinstance(k.key, str):
                 text_count += 1
             if k.key == Keys.ControlM:
                 newline_count += 1
@@ -303,27 +328,52 @@ class ConsoleInputReader(object):
         result = None
 
         u_char = ev.uChar.UnicodeChar
-        ascii_char = u_char.encode('utf-8')
+        ascii_char = u_char.encode("utf-8")
 
         # NOTE: We don't use `ev.uChar.AsciiChar`. That appears to be latin-1
         #       encoded. See also:
         # https://github.com/ipython/ipython/issues/10004
         # https://github.com/jonathanslenders/python-prompt-toolkit/issues/389
 
-        if u_char == '\x00':
+        if u_char == "\x00":
             if ev.VirtualKeyCode in self.keycodes:
-                result = KeyPress(self.keycodes[ev.VirtualKeyCode], '')
+                result = KeyPress(self.keycodes[ev.VirtualKeyCode], "")
         else:
             if ascii_char in self.mappings:
                 if self.mappings[ascii_char] == Keys.ControlJ:
-                    u_char = '\n'  # Windows sends \n, turn into \r for unix compatibility.
+                    u_char = (
+                        "\n"  # Windows sends \n, turn into \r for unix compatibility.
+                    )
                 result = KeyPress(self.mappings[ascii_char], u_char)
             else:
                 result = KeyPress(u_char, u_char)
 
-        # Correctly handle Control-Arrow keys.
-        if (ev.ControlKeyState & self.LEFT_CTRL_PRESSED or
-                ev.ControlKeyState & self.RIGHT_CTRL_PRESSED) and result:
+        # First we handle Shift-Control-Arrow/Home/End (need to do this first)
+        if (
+            (
+                ev.ControlKeyState & self.LEFT_CTRL_PRESSED
+                or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
+            )
+            and ev.ControlKeyState & self.SHIFT_PRESSED
+            and result
+        ):
+            if result.key == Keys.Left:
+                result.key = Keys.ShiftControlLeft
+
+            if result.key == Keys.Right:
+                result.key = Keys.ShiftControlRight
+
+            if result.key == Keys.Home:
+                result.key = Keys.ShiftControlHome
+
+            if result.key == Keys.End:
+                result.key = Keys.ShiftControlEnd
+
+        # Correctly handle Control-Arrow/Home/End and Control-Insert keys.
+        if (
+            ev.ControlKeyState & self.LEFT_CTRL_PRESSED
+            or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
+        ) and result:
             if result.key == Keys.Left:
                 result.key = Keys.ControlLeft
 
@@ -336,22 +386,64 @@ class ConsoleInputReader(object):
             if result.key == Keys.Down:
                 result.key = Keys.ControlDown
 
+            if result.key == Keys.Home:
+                result.key = Keys.ControlHome
+
+            if result.key == Keys.End:
+                result.key = Keys.ControlEnd
+
+            if result.key == Keys.Insert:
+                result.key = Keys.ControlInsert
+
         # Turn 'Tab' into 'BackTab' when shift was pressed.
+        # Also handle other shift-key combination
         if ev.ControlKeyState & self.SHIFT_PRESSED and result:
             if result.key == Keys.Tab:
                 result.key = Keys.BackTab
 
+            if result.key == Keys.Left:
+                result.key = Keys.ShiftLeft
+
+            if result.key == Keys.Right:
+                result.key = Keys.ShiftRight
+
+            if result.key == Keys.Up:
+                result.key = Keys.ShiftUp
+
+            if result.key == Keys.Down:
+                result.key = Keys.ShiftDown
+
+            if result.key == Keys.Home:
+                result.key = Keys.ShiftHome
+
+            if result.key == Keys.End:
+                result.key = Keys.ShiftEnd
+
+            if result.key == Keys.Insert:
+                result.key = Keys.ShiftInsert
+
         # Turn 'Space' into 'ControlSpace' when control was pressed.
-        if (ev.ControlKeyState & self.LEFT_CTRL_PRESSED or
-                ev.ControlKeyState & self.RIGHT_CTRL_PRESSED) and result and result.data == ' ':
-            result = KeyPress(Keys.ControlSpace, ' ')
+        if (
+            (
+                ev.ControlKeyState & self.LEFT_CTRL_PRESSED
+                or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
+            )
+            and result
+            and result.data == " "
+        ):
+            result = KeyPress(Keys.ControlSpace, " ")
 
         # Turn Control-Enter into META-Enter. (On a vt100 terminal, we cannot
         # detect this combination. But it's really practical on Windows.)
-        if (ev.ControlKeyState & self.LEFT_CTRL_PRESSED or
-                ev.ControlKeyState & self.RIGHT_CTRL_PRESSED) and result and \
-                result.key == Keys.ControlJ:
-            return [KeyPress(Keys.Escape, ''), result]
+        if (
+            (
+                ev.ControlKeyState & self.LEFT_CTRL_PRESSED
+                or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
+            )
+            and result
+            and result.key == Keys.ControlJ
+        ):
+            return [KeyPress(Keys.Escape, ""), result]
 
         # Return result. If alt was pressed, prefix the result with an
         # 'Escape' key, just like unix VT100 terminals do.
@@ -365,7 +457,7 @@ class ConsoleInputReader(object):
             meta_pressed = ev.ControlKeyState & self.LEFT_ALT_PRESSED
 
             if meta_pressed:
-                return [KeyPress(Keys.Escape, ''), result]
+                return [KeyPress(Keys.Escape, ""), result]
             else:
                 return [result]
 
@@ -384,69 +476,108 @@ class ConsoleInputReader(object):
         if ev.ButtonState == FROM_LEFT_1ST_BUTTON_PRESSED:
             # On a key press, generate both the mouse down and up event.
             for event_type in [MouseEventType.MOUSE_DOWN, MouseEventType.MOUSE_UP]:
-                data = ';'.join([
-                   event_type,
-                   str(ev.MousePosition.X),
-                   str(ev.MousePosition.Y)
-                ])
+                data = ";".join(
+                    [str(event_type), str(ev.MousePosition.X), str(ev.MousePosition.Y)]
+                )
                 result.append(KeyPress(Keys.WindowsMouseEvent, data))
 
         return result
 
 
-_current_callbacks = {}  # loop -> callback
+class _Win32Handles:
+    """
+    Utility to keep track of which handles are connectod to which callbacks.
+    """
+
+    def __init__(self) -> None:
+        self._handle_callbacks: Dict[int, Callable[[], None]] = {}
+
+    def add_win32_handle(self, handle: HANDLE, callback: Callable[[], None]) -> None:
+        """
+        Add a Win32 handle to the event loop.
+        """
+        if handle.value is None:
+            raise ValueError("Invalid handle.")
+        handle_value = handle.value
+
+        loop = get_event_loop()
+        self._handle_callbacks[handle_value] = callback
+
+        # Add reader.
+        def ready() -> None:
+            # Tell the callback that input's ready.
+            try:
+                callback()
+            finally:
+                run_in_executor_with_context(wait, loop=loop)
+
+        # Wait for the input to become ready.
+        # (Use an executor for this, the Windows asyncio event loop doesn't
+        # allow us to wait for handles like stdin.)
+        def wait() -> None:
+            if self._handle_callbacks.get(handle_value) != callback:
+                return
+
+            wait_for_handles([handle])
+            loop.call_soon_threadsafe(ready)
+
+        run_in_executor_with_context(wait, loop=loop)
+
+    def remove_win32_handle(self, handle: HANDLE) -> None:
+        """
+        Remove a Win32 handle from the event loop.
+        """
+        if handle.value in self._handle_callbacks:
+            del self._handle_callbacks[handle.value]
 
 
 @contextmanager
-def attach_win32_input(input, callback):
+def attach_win32_input(input: _Win32InputBase, callback: Callable[[], None]):
     """
     Context manager that makes this input active in the current event loop.
 
     :param input: :class:`~prompt_toolkit.input.Input` object.
     :param input_ready_callback: Called when the input is ready to read.
     """
-    assert isinstance(input, Input)
-    assert callable(callback)
+    win32_handles = input.win32_handles
+    handle = input.handle
 
-    loop = get_event_loop()
-    previous_callback = _current_callbacks.get(loop)
+    if handle.value is None:
+        raise ValueError("Invalid handle.")
 
     # Add reader.
-    loop.add_win32_handle(input.handle, callback)
-    _current_callbacks[loop] = callback
+    previous_callback = win32_handles._handle_callbacks.get(handle.value)
+    win32_handles.add_win32_handle(handle, callback)
 
     try:
         yield
     finally:
-        loop.remove_win32_handle(input.handle)
+        win32_handles.remove_win32_handle(handle)
 
         if previous_callback:
-            loop.add_win32_handle(input.handle, previous_callback)
-            _current_callbacks[loop] = previous_callback
-        else:
-            del _current_callbacks[loop]
+            win32_handles.add_win32_handle(handle, previous_callback)
 
 
 @contextmanager
-def detach_win32_input(input):
-    assert isinstance(input, Input)
+def detach_win32_input(input: _Win32InputBase):
+    win32_handles = input.win32_handles
+    handle = input.handle
 
-    loop = get_event_loop()
-    previous = _current_callbacks.get(loop)
+    if handle.value is None:
+        raise ValueError("Invalid handle.")
 
+    previous = win32_handles._handle_callbacks.get(handle.value)
     if previous:
-        loop.remove_win32_handle(input.handle)
-        _current_callbacks[loop] = None
+        win32_handles.remove_win32_handle(handle)
 
     try:
         yield
     finally:
         if previous:
-            loop.add_win32_handle(input.handle, previous)
-            _current_callbacks[loop] = previous
+            win32_handles.add_win32_handle(handle, previous)
 
 
-class raw_mode(object):
+class raw_mode:
     """
     ::
 
@@ -456,8 +587,9 @@ class raw_mode(object):
     The ``fileno`` attribute is ignored. This is to be compatible with the
     `raw_input` method of `.vt100_input`.
     """
+
     def __init__(self, fileno=None):
-        self.handle = windll.kernel32.GetStdHandle(STD_INPUT_HANDLE)
+        self.handle = HANDLE(windll.kernel32.GetStdHandle(STD_INPUT_HANDLE))
 
     def __enter__(self):
         # Remember original mode.
@@ -467,17 +599,19 @@ class raw_mode(object):
 
         self._patch()
 
-    def _patch(self):
+    def _patch(self) -> None:
         # Set raw
         ENABLE_ECHO_INPUT = 0x0004
         ENABLE_LINE_INPUT = 0x0002
         ENABLE_PROCESSED_INPUT = 0x0001
 
         windll.kernel32.SetConsoleMode(
-            self.handle, self.original_mode.value &
-            ~(ENABLE_ECHO_INPUT | ENABLE_LINE_INPUT | ENABLE_PROCESSED_INPUT))
+            self.handle,
+            self.original_mode.value
+            & ~(ENABLE_ECHO_INPUT | ENABLE_LINE_INPUT | ENABLE_PROCESSED_INPUT),
+        )
 
-    def __exit__(self, *a, **kw):
+    def __exit__(self, *a: object) -> None:
         # Restore original mode
         windll.kernel32.SetConsoleMode(self.handle, self.original_mode)
 
@@ -489,12 +623,15 @@ class cooked_mode(raw_mode):
         with cooked_mode(stdin):
             ''' The pseudo-terminal stdin is now used in cooked mode. '''
     """
-    def _patch(self):
+
+    def _patch(self) -> None:
         # Set cooked.
         ENABLE_ECHO_INPUT = 0x0004
         ENABLE_LINE_INPUT = 0x0002
         ENABLE_PROCESSED_INPUT = 0x0001
 
         windll.kernel32.SetConsoleMode(
-            self.handle, self.original_mode.value |
-            (ENABLE_ECHO_INPUT | ENABLE_LINE_INPUT | ENABLE_PROCESSED_INPUT))
+            self.handle,
+            self.original_mode.value
+            | (ENABLE_ECHO_INPUT | ENABLE_LINE_INPUT | ENABLE_PROCESSED_INPUT),
+        )
