@@ -1,12 +1,10 @@
 """
 Logging configuration.
 """
-from __future__ import unicode_literals
-
 import logging
 
 __all__ = [
-    'logger',
+    "logger",
 ]
 
 logger = logging.getLogger(__package__)
